/**********************************************************************
*
* vdoxs.h
*
* shared header for ctxping/vdping example program
*
* Copyright (C) Citrix Systems, Inc.  All Rights Reserved.
*
* The code and comments contained in this file are provided "as is"
* without warranty of any kind, either expressed or implied,
* including, but not limited to, the implied warranties of
* merchantability and/or fitness for a particular purpose.
* You have the right to use, modify, reproduce, and distribute
* this file, and/or any modified version of this file, in any way
* that you find useful, provided that you agree that Citrix Systems
* has no warranty obligations or liability for this or any other
* sample file.
*
**********************************************************************/

#ifndef __VDOXS_H__
#define __VDOXS_H__

#include <ica.h>
#include <ica-c2h.h>        /* for VD_C2H structure */

#define CTXOXS_VIRTUAL_CHANNEL_NAME  "CTXOXS"
#define VC_TIMEOUT_MILLISECONDS     10000L

/*
 * Lowest and highest compatable version.  See DriverInfo().
 */
#define CTXOXS_VER_LO      1
#define CTXOXS_VER_HI      1

/* Noughts & Crosses grid size */
#define OXS_GRID_WIDTH 3
#define OXS_GRID_HEIGHT 3
#define OXS_SQUARE_SIZE 100
#define OXS_HALF_SQUARE 50
#define OXS_QUARTER_SQUARE 25

#pragma pack(1)


/* vdoxs driver info (client to host) structure */
typedef struct _VDOXS_C2H
{
    VD_C2H  Header;
    USHORT  usMaxDataSize;      /* maximum data block size */
} VDOXS_C2H, * PVDOXS_C2H;


/* vdoxs server to client packet types */
enum packetType {
    OXS_PARENT_WINDOW,
    OXS_PLAY_MOVE,
    OXS_WINNING_LINE,
    OXS_MOVE_WINDOW,
    OXS_CLOSE_WINDOW
};


/* Two types of counter */
enum counter {
    NONE,
    NOUGHT,
    CROSS
};


/* Header structure */
typedef struct _OXS_HEADER
{
    enum packetType type;
} OXS_HEADER, * POXS_HEADER;


typedef struct _OXS_COORDINATE
{
    UINT32   x;
    UINT32   y;
} OXS_COORDINATE, * POXS_COORDINATE;


typedef struct _OXS_PARENT_WINDOW_PACKET
{
    UINT32 parent;
} OXS_PARENT_WINDOW_PACKET, * POXS_PARENT_WINDOW_PACKET;


typedef struct _OXS_PLAY_MOVE_PACKET
{
    OXS_COORDINATE position;
    enum counter counter;
} OXS_PLAY_MOVE_PACKET, * POXS_PLAY_MOVE_PACKET;


typedef struct _OXS_WINNING_LINE_PACKET
{
    OXS_COORDINATE startpos;
    OXS_COORDINATE finishpos;
} OXS_WINNING_LINE_PACKET, * POXS_WINNING_LINE_PACKET;


typedef OXS_COORDINATE OXS_MOVE_WINDOW_PACKET, * POXS_MOVE_WINDOW_PACKET;


#pragma pack()

#endif /* __VDOXS_H__ */
